//////////////////////////////////////////////////////////////////
//
// File: raycast.cpp
// Author: Sven Goossens
// Purpose:
//		This file contains the function implementation of the 
//		function that calculates the distance to the intersection
//		of a ray and a wall.
//
//////////////////////////////////////////////////////////////////

#include "raycast.h"
#define _USE_MATH_DEFINES
#include <math.h>

const double DEG2RAD = (M_PI/180.f);

//---------------------------------------------------------------------------
/*
	float IntersectLen(float angle, int* viewpoint, int* p1, int* p2)
	This function takes 3 arguments:
		float angle:	the angle in degrees at which the ray is fired. An angle of 0 corresponds
						a ray positive x-direction
		int* viewpoint: A pointer to a 2 dimensional array, containing the x- and y-coordinate of
						the player
		int* p1:		The first coordinate-pair of the wall
		int* p2:		The second coordinate-pair of the wall
*/
float IntersectLen(float angle, int* viewpoint, int* p1, int* p2){
	int x1, x2, y1, y2;
	float s1, s2, x, y, dist;

	// Swap coordinates so we are always looking in the positive x direction
	if(p1[0] <= p2[0]){
		x1 = p1[0];
		y1 = p1[1];
		x2 = p2[0];
		y2 = p2[1];
	}
	else{
		x1 = p2[0];
		y1 = p2[1];
		x2 = p1[0];
		y2 = p1[1];
	}

	// Clip the angle to a value between 0 and 360 degrees
	if(angle > 0)
		angle = angle-360*int(angle/360.f);
	else
		angle = angle-360*int(angle/360.f);

	if(angle < 0)
		angle = 360+angle;

	s1 = tan((angle)*(float)DEG2RAD); // slope of the ray

	if((x2-x1) != 0) {
		s2 = (float)(y2-y1)/(x2-x1);									// slope of the wall
		if((s2-s1) != 0){
			x = (viewpoint[1]-y1-viewpoint[0]*s1+x1*s2)/(s2-s1);	// Intersection point x
			y = (x-x1)*s2+y1;										// Intersection point y
		}
		else
			return -1;	// parallel lines
	}
	else{
		x = (float)x1;									// Special case, vertical line
		y = (x-viewpoint[0])*s1+viewpoint[1];	// calculate the y intersection point
	}

	// Determine if the intersection point is actually on the wall
	if(between((int)x, x1, x2) && between((int)y, y1, y2)){
		// intersection is on the wall line
		dist = sqrt((y-viewpoint[1])*(y-viewpoint[1])+(x-viewpoint[0])*(x-viewpoint[0]));

		// Is the ray fired in the direction of the intersection?
		if(angle >= 0 && angle < 90){
			if(x>viewpoint[0] && y>viewpoint[1]){
				return dist;
			}
		}
		if(angle >= 90 && angle < 180){
			if(x<viewpoint[0] && y>viewpoint[1]){
				return dist;
			}
		}
		if(angle >= 180 && angle < 270){
			if(x<viewpoint[0] && y<viewpoint[1]){
				return dist;
			}
		}
		if(angle >= 270 && angle < 360){
			if(x>viewpoint[0] && y<viewpoint[1]){
				return dist;
			}
		}
	}
	// not on wall
	return -1;
}
//---------------------------------------------------------------------------
/*
	int between(int num, int border1, int border2)
	This is a helper function for the intersectLen function,
	that checks if a certain number num lies between
	border1 and border2.
*/

int between(int num, int border1, int border2){
	if(border2 >= border1)
		if(num >= border1 && num <= border2)
			return 1;
		else
			return 0;
	else
		if(num >= border2 && num <= border1)
			return 1;
		else
			return 0;
}

